;;;
;;; <daemon.ss> ---- Servidor del SisMo
;;;
;;; Copyright (C) 2001 by Karla Ramrez. 
;;;
;;; This file is part of SisMo.

;;; SisMo is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Lesser General Public
;;; License as published by the Free Software Foundation; either
;;; version 2.1 of the License, or (at your option) any later version.

;;; SisMo is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Lesser General Public License for more details.

;;; You should have received a copy of the GNU Lesser General Public
;;; License along with SisMo; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Author: Karla Ramrez <karla@abulafia.fciencias.unam.mx>
;;
;;
;; Commentary:

;; Libraries:
;; **********
;; SchemeQL
;; **********
(define odbc-version 2.5)
(require-library "schemeql.ss" "schemeql")
(schemeql-verbose-level 10)

;; local:
(load-relative "util.ss")
(load-relative "private.ss")

;; Configuracin:
(load-relative "config.ss")

;; Log information:
(define log-port (open-output-file (sismo-log) 'append))
(current-error-port log-port)

;; Starting up!
(log-this "notice" "SisMo starting")

;; Opening conncetion
(connect-to-database dbms login passwd)
(set! dbase (use-database database))
(log-this "notice" "Connection to database opened")

(define reconnect-now
  (lambda ()
    ;; Re-opening conncetion
    (connect-to-database dbms login passwd)))

;; Tiempo que esperamos los datos completos de un host:
(define max-time-wait 180)
(define time-left
  (let ((c (current-seconds)))
    (lambda () (- max-time-wait (- (current-seconds) c)))))

;; Procedures:
; sismo-serve : [Nat] [Nat] -> Void
; to start the server on the given port and return an un-server to
; shut it down max-waiting is the maximum number of clients waiting
; for the acceptance of their connection.
(define sismo-serve
  (opt-lambda
   ([port (sismo-port)] 
    [max-waiting (sismo-max-waiting-clients)])
    (let ([custodian (make-custodian)])
      (parameterize ([current-custodian custodian])
		    (let ([listener (tcp-listen port max-waiting)])
		      ;; If tcp-listen fails, the exception will be raised in the caller's thread.
		      (thread
		       (lambda ()
			 (server-loop listener)))))
      (lambda () (custodian-shutdown-all custodian)))))

;server-loop : Tcp-listener -> Void
(define (server-loop listener)
  (let-values ([(ip op) (tcp-accept listener)])
	      (let ([cust (make-custodian)])
		(parameterize ([current-custodian cust]
			       [exit-handler (lambda (x) (custodian-shutdown-all cust))])
			      (thread (lambda ()
					(sismo-serve-connection ip op)
					(custodian-shutdown-all cust))))))
  (server-loop listener))

;; La siguiente lista contiene a todas las tablas en el SisMo que
;; slo pueden recibir una entrada a la vez.  El resto de las tablas
;; pueden recibir varias entradas con el mismo timestamp.
(define only-once '("process" "processor" "memory" "nfs"))

;; serve-connection : input-port output-port -> void
(define sismo-serve-connection
  (lambda (ip op)
    ;; Esto nos sirve para saber si el servidor de MySQL ya cerr
    ;; la coneccin o no.  En caso de haberla cerrado, la volvemos
    ;; a abrir y continuamos.
    (with-handlers
     ([general-sql-error? (lambda (x)
			    (log-this "exception"
				      (format "~a"
					      (general-sql-error-diagnostics x)))
			    (reconnect-now)
			    (use-database database))])
     (use-database database))
    (let ((data? (make-parameter #f))
	  (current-id (make-parameter 0))
	  (current-kid (make-parameter "")))

      (dynamic-wind
       void
       ;; Este es el ciclo principal para manejar el protocolo B
       ;; de los clientes en los hosts siendo monitoreados.
       ;; cliente:    "hello:kid:host:ts"
       ;; servidor:   if OK "olleh" else "error"
       ;; cliente:    data [:data ...]
       ;; cliente:    "."
       ;; servidor:   "bye"
	 
       ;; Tambin includo aqu estn las funciones para manejar
       ;; el protocolo A, slo que esos son ms sencillos y no hacen
       ;; "loop", simplemente guardan la informacin en las tablas
       ;; apropiadas y regresan "bye" al cliente.
       (lambda ()
	 (let loop ((ln (read-line ip)))
	   (if (eof-object? ln)
	       ;; Cerramos la conexin
	       (when (data?)
		     ;; Warning: esto no es usual, se cay el cliente?
		     (log-this "warning" "El cliente envi EOF prematuramente."))
	       ;; OK, seguimos atendiendo al cliente:
	       (let* ((lst (string-tokenizer #\: ln))
		      (command (car lst)))
		 (cond (;; Fin del protocolo B:
			(string=? command ".")
			(display "bye" op)
			(newline op))
		       (;; Modo de lectura de datos (se supone que
			;; ya recibimos un "hello kid host timestamp"
			;; antes de empezar a recibir datos.
			(data?)
			(store-data (current-id) (current-kid) lst)
			(when (member (current-kid) only-once)
			      ;; Dile al cliente adios:
			      (display "bye" op)
			      (newline op))
			;; Keep reading:
			(loop (read-line ip))
			)
		       (;; Inicio del protocolo B:
			(string=? command "hello")
			(let* ((kid (list-ref lst 1))
			       (host (list-ref lst 2))
			       (ts (list-ref lst 3))
			       (hostid (is-host-registered host)))
			  (cond (;; Primero verificamos que este host,
				 ;; sea uno que "s" estamos monitoreando:
				 (> hostid 0)
				 ;; Almacenamos la informacin en las bases:
				 (current-id (store-id kid hostid ts))
				 ;; Recordamos el KID actual:
				 (current-kid (cadr lst))
				 ;; Y le decimos al cliente que todo est bien:
				 (display "olleh" op)
				 (newline op)
				 ;; Activa el modo de lectura de datos:
				 (data? #t)
				 (loop (read-line ip)))
				;; Host desconocido:
				(else
				 ;; Guardamos este error en el log:
				 (log-this "error"
					   (format "Host desconocido: ~a" host))
				 (write "Error: Host desconocido" op)
				 (newline op)))))
		       ;; El protocolo A:
		
		       (;; Para aadir un host al SisMo (esto slo lo
			;; puede hacer un admin que ya est logeado en el
			;; sistema):
			(string=? command "addhost")
			(let* ((adminid (list-ref lst 1))
			       (host-ip (list-ref lst 2))
			       (hostname (list-ref lst 3))
			       ;; Primero aadimos el host a la tabla de hosts:
			       (hostid (store-host host-ip hostname)))
			  ;; Ya sea que el host ya se encontrara en la
			  ;; base de datos o no (y por ende haya sido
			  ;; aadido).  Tenemos que aadir la
			  ;; relacin: administrador-host.
			  (insert-row (get-table "admin_host")
				      `((adminid ,adminid) (hostid ,hostid)))
			  (write "ok" op)
			  (newline op)))
		       (;; 
			(string=? command "searchhost")
			(let ((data (list-ref lst 1))
			      (search-by (string->symbol (list-ref lst 2))))
			  (write (search-host data search-by) op)
			  (newline op)))
		       (;; Para borrar un host del sistema
			(string=? command "deletehost")
			(let ((hostid (string->number (list-ref lst 1))))
			  ;; Borramos el host (pero no borrarmos toda la informacin
			  ;; que ya tenemos sus "clientes" en las tablas.  Por si acaso
			  ;; el admin se arrepiente o algo as.
			  (delete (get-table "hosts") `((= hostid ,hostid)))
			  ;; Y tambin borramos de la tabla que relaciona al host con
			  ;; los admins:
			  (delete (get-table "admin_host") `((= hostid ,hostid)))
			  (write "ok" op)
			  (newline op)))
		       (;; Para aadir un nuevo administrador al sistema.
			(string=? command "addadmin")
			(let* ((login (list-ref lst 1))
			       (name  (list-ref lst 2))
			       ;; Primero vemos si ya est ese login dado de alta:
			       (here (search-admin login 'login)))
			  (if (null? here)
			      (begin
				;; Perfecto continuamos:
				;; regresamos su password.  Ms adelante, l/ella mismo(a)
				;; lo podr cambiar.
				(write (add-admin login name) op)
				(newline op))
			      (begin
				;; Algo sali mal, regresamos un error:
				(write "error: el login ya existe " op)
				(newline op)))))
		       (;; Para buscar a un administrador
			(string=? command "searchadmin")
			(let ((data (list-ref lst 1))
			      (search-by (string->symbol (list-ref lst 2))))
			  (write (search-admin data search-by) op)
			  (newline op)))
		       (;; Para borrar un admin del sistema
			(string=? command "deleteadmin")
			(let ((adminid (string->number (list-ref lst 1))))
			  ;; Borramos al admin:
			  (delete (get-table "admins") `((= adminid ,adminid)))
			  ;; Y tambin borramos de la tabla que relaciona al host con
			  ;; los admins:
			  (delete (get-table "admin_host") `((= adminid ,adminid)))
			  (write "ok" op)
			  (newline op)))
		       (;; Cambiar password:
			(string=? command "changepasswd")
			(let* ((adminid (string->number (list-ref lst 1)))
			       (old (list-ref lst 2))
			       (new (list-ref lst 3))
			       ;; Es correcto el password viejo?
			       (cursor (find '(admins) '(login name)
					     `((= adminid ,adminid)
					       (= passwd ,old))))
			       )
			  (if (not (cursor-null? cursor))
			      (if (is-password-cool new)
				  (begin
				    ;; Hacemos un update a la tabla de admins y
				    ;; presto!, el administrador podr usar su nuevo
				    ;; password:
				    (update "admins" `((passwd ,new)) `((= adminid ,adminid)))
				    (write "OK: El password ha sido cambiado" op)
				    (newline op))
				  (begin (write "Error: El nuevo password no es seguro." op)
					 (newline op)))
			      (begin (write "Error: El password ACTUAL es incorrecto." op)
				     (newline op)))))
		       (;; Obtener todos los hosts que administra un administrador:
			(string=? command "getmyhosts")
			(let* ((adminid (string->number (list-ref lst 1))))
			  (write (get-hosts-admin-by adminid) op)
			  (newline op)))
		       (;; login
			(string=? command "login")
			(let* ((login (list-ref lst 1))
			       (passwd (list-ref lst 2))
			       (admin (find '(admins) '(adminid name)
					    `((= login ,login) (= passwd ,passwd)))))
			  (if (cursor-null? admin)
			      (write "error" op)
			      (write (map cadr (get-row admin)) op))
			  (newline op)))
		       
		       ;; AQUI vamos (falta ms funciones del protocolo A:
		       ;;  - Eliminar un host
		       ;;  - Eliminar un admin

		       ;; Esta es la opcin que hace toda la talacha
		       ;; de convertir los datos (bueno, parte de
		       ;; ellos) que envan los clientes para poderlos
		       ;; graficar.
		       ((string=? command "get")
			(let* ((hostid (list-ref lst 1))
			       (area (list-ref lst 2))
			       (from (list-ref lst 3))
			       (to (list-ref lst 4))
			       (kids (get-kids-range hostid area from to))
			       (x-labels (generate-x-labels kids))
			       (rows '()))
			  (for-each (lambda (n)
				      (let ((c (get-data-from area n)))
					(map (lambda (data)
					       (let* ((label (car data))
						      (val (cdr data))
						      (lst (assoc label rows)))
						 (if lst
						     (set-graph-data!
						      (cadr lst)
						      (append (graph-data (cadr lst)) val))
						     (set! rows
							   (append rows
								   (list (list label (make-graph val))))))))
					     c)))
				    kids)
			  ;; Y regresamos una lista con todos los
			  ;; datos necesarios para poder graficar:
			  (write (list (length kids) ;; Number of points (X)
				       (length rows) ;; Number of lines (Y)
				       (map car rows) ;; Etiquetas
				       (map (lambda (r)
					      (graph-data (cadr r))) rows) ;; Lneas
				       x-labels) ;; x-labels
				 op)
			  (newline op)))

		       ;; Una vez que un admin est viendo una
		       ;; grfica.  Puede decidir ver los datos que
		       ;; nos llevaron a graficar cada una de las
		       ;; lneas.
		       ((string=? command "gettable")
			(let* ((hostid (list-ref lst 1))
			       (area (list-ref lst 2))
			       (label (list-ref lst 3))
			       (from (list-ref lst 4))
			       (to (list-ref lst 5))
			       (kids (get-kids-range hostid area from to))
			       (rows '()))
			  
			  (write
			   ;; Agregamos las etiquetas que mostrar el SisMo.
			   (cons
			    (cond ((string=? area "fs")
				   '("TimeStamp" "Bloques Usados" "Bloques Disponibles" "% de uso"))
				  ((string=? area "processor")
				   '("TimeStamp" "Carga" "User time" "System time" "Idle time"))
				  ((string=? area "process")
				   '("TimeStamp" "Demonios" "de Sistema" "de Usuario" "Corriendo" "Durmiendo" "Zombies"))
				  ((string=? area "memory")
				   '("TimeStamp" "usada" "libre" "compartida" "cache" "swap usada" "swap libre"))
				  ((string=? area "users")
				   '("TimeStamp" "usuario" "TTY" "desde" "ejecutando" "at"))
				  ((string=? area "nfs")
				   '("TimeStamp" "importados" "exportados" "montados"))
				  ((string=? area "printer")
				   '("TimeStamp" "impresora" "encolados" "imprimiendo" "total"))
				  ((string=? area "network")
				   '("TimeStamp" "interfaz" "(i)recibidos" "(i)errores" "(i)tirados" "(i)overrun"
				     "(o)enviados" "(o)errores" "(o)tirados" "(o)overrun"))
				  )
			    (let loop ((kids kids))
			      (cond ((null? kids) ())
				    ((or (string=? area "users")
					 (string=? area "printer")
					 (string=? area "network"))
				     (append (get-raw-data area label (car kids))
					     (loop (cdr kids))))
				    (else
				     ;; Nota: El car extra que tenemos
				     ;; que sacar aqu, es porque
				     ;; algunos clientes no regresan
				     ;; ms que un elemento y eso debe
				     ;; estar slo dentro de dos
				     ;; listas (no tres).
				     (cons (car (get-raw-data area label (car kids)))
					   (loop (cdr kids))))))
			    ;; (map (lambda (n)
			    ;; 	      (let ((res (get-raw-data area label n)))
			    ;; 		(log-this "debug" (format "Round (~a): ~a~n" n res))
			    ;; 		;; FIXME: Este car tal vez no debera hacerse en
			    ;; 		;; todos los casos.  Las sub-listas deben ser de
			    ;; 		;; la forma: ("dato1" "dato2" ... "datoN").
			    ;; 		(car res)))
			    ;; 	    kids)
			    )
			   op)
			  (newline op)))
		       
		       ;; Nada caso, entonces mandamos un error:
		       (else
			(write "error" op)
			(newline op)
			(log-this "error"
				  (format "Comando desconocido: ~a" command))))))))
       ;; Tercera parte del dynamic wind:
       (lambda ()
	 (close-input-port ip)
	 (close-output-port op))))))

;; *********************************
;;   Funciones para el protocolo A 
;; *********************************

;; add-admin : String String -> String
;; Recibe el login y el nombre completo de un administrador,
;; genera un password aleatorio para ese administrador,
;; guarda todos los datos en la tabla admins y regresa
;; el password recin generado.
(define add-admin
  (lambda (login name)
    (let ((passwd (create-dummy-password)))
      ;; Guardamos todo en la tabla admins:
      (insert-row (get-table "admins")
		  `((login ,login) (name ,name) (passwd ,passwd)))
      ;; Regresamos el password temporal:
      passwd)))

;; search-admin : String Symbol -> List
;; Dependiendo del valor de search-by ('id, 'login, 'name)
;; hace un query en la base de administradores y regresa todos
;; los administradores que casen con la bsqueda.
(define search-admin
  (lambda (data search-by)
    (let ((cursor (find '(admins) '(adminid login name)
			(case search-by
			  ((id)
			   `((= adminid ,(string->number data))))
			  ((login)
			   `((= login ,data)))
			  (else ;; nombre:
			   `((like name ,(format "%~a%" data))))))))
      (if (cursor-null? cursor)
	  ()
	  (finite-cursor->list
	   (cursor-map (lambda (row)
			 (map cadr row))
		       cursor))))))

;; get-hosts-admin-by: Nat -> (listof hosts)
;; Regresa todos los hosts que son administrados actualmente por el
;; administrador con ID: adminid.
(define get-hosts-admin-by
  (lambda (adminid)
    (let ((cursor (find '(hosts admin_host)
			'((hosts hostid) (hosts ip) (hosts hostname))
			`((= (admin_host adminid) ,adminid)
			  (= (admin_host hostid) (hosts hostid))))))
      (finite-cursor->list
       (cursor-map (lambda (row)
		     (map cadr row)) cursor)))))


;; search-host : String Symbol -> List
;; Dependiendo del valor de search-by ('id, 'ip, 'name)
;; hace un query en la base de hosts y regresa todos
;; los que casen con la bsqueda.
(define search-host
  (lambda (data search-by)
    (let ((cursor (find '(hosts) '(hostid ip hostname)
			(case search-by
			  ((id)
			   `((= hostid ,(string->number data))))
			  ((ip)
			   `((= ip ,data)))
			  (else ;; nombre:
			   `((like hostname ,(format "%~a%" data))))))))
      (if (cursor-null? cursor)
	  ()
	  (finite-cursor->list
	   (cursor-map (lambda (row)
			 (map cadr row))
		       cursor))))))

;; store-host : String String -> Nat
;; Si el host con IP y nombre HOSTNAME an no est registrado en
;; la base de datos, lo agrega y regresa su hostid asignado, en
;; caso contrario regresa 0 (cero).
(define store-host
  (lambda (ip hostname)
    (let ((id (is-host-registered hostname)))
      (cond ((zero? id)
	     ;; An no tenemos registrado este host, as que lo agregamos
	     ;; en este momento.
	     (insert-row (get-table "hosts")
			 `((ip ,ip) (hostname ,hostname)))
	     ;; Obtenemos el nuevo hostid:
	     (cadar (get-row (find 'last_insert_id))))
	    (else
	     ;; Ya estaba registrado, slo regresamos su id.
	     id)))))

;; is-host-registered: String -> Nat
;; Si la cadena (una direccin IP) corresponde a un host
;; siendo monitoreado, entonces regresa el `hostid' (un
;; nmero natural) asignado en la tabla `hosts'.  Regresa
;; cero si el host no est registrado con el SisMo.
(define is-host-registered
  (lambda (h)
    (let ((cursor (find '(hosts) '(hostid) `((= hostname ,h)))))
      (if (cursor-null? cursor)
	  ;; Sorry, pero no, este no es un host monitoreado:
	  0
	  ;; El primer rengln del cursor luce como:
	  ;;       (("hostid" N))
	  ;; por eso sacamos el hostid as:
	  (cadar (get-row cursor))))))

;; store-id : List -> Natural
;; Guarda los datos en la tabla kid y regresa el `id' correspondiente
;; a dichos datos.
(define store-id
  (lambda (kid hostid ts)
    ;; Guarda la informacin en la tabla `kid'.
    (insert-row (get-table "kid")
		`((kid ,kid) (hostid ,hostid) (ts ,ts)))
    ;; Regresamos el id recin creado en la tabla:
    (cadar (get-row (find 'last_insert_id)))))


;; Tablas para el manejo de administradores y host (protocolo A)
(define hosts-alist '(hostid ip hostname))
(define admins-alist '(adminid login name passwd))
(define admin_host-alist '(adminid hostid))

;; ********************************
;;   Funciones para el protocolo B
;; ********************************

;; Listas de nombres de campos de cada tabla en el SisMo (protocolo B)
(define users-alist '(user tty remote process sessions login_at))
(define process-alist '(total demons system user run sleep zombie))
(define processor-alist '(load_avg user system idle))
(define fs-alist '(fs blocks used available percent mountat))
(define memory-alist '(total used free shared cached swap s_used s_free))
(define network-alist '(interface bytes_r packets_r errors_r drops_r overs_r
				  bytes_t packets_t errors_t drops_t overs_t
				  colls))
(define nfs-alist '(imported exported mounted))
(define printer-alist '(printer queueing printing entries))

;; store-data : Natural String List -> void
;; Esta funcin recibe el identificador (nico) del proceso
;; que recin se conecto para almacenar informacin, el "nombre"
;; del proceso (users, process, etc.) y la lista de los valores
;; correspondientes a dicho proceso.
;; store-data busca la lista del proceso, que tiene los nombres
;; de cada uno de los campos en la tabla y despus apareja estos
;; nombres con los datos que enva el host y los guarda en la
;; tabla correspondiente.
(define store-data
  (lambda (id kid lst)
    (insert-row (get-table kid)
		(cons (list 'id id)
		      (map (lambda (name value)
			     (list name value))
			   (eval (string->symbol
				  (string-append kid "-alist")))
			   lst)))))

;; get-kids-range: Nat String String String -> (listof Nat)
;; Regresa una lista con los KIDs que estan en el rango entre
;; from y to (dos cadenas que representan time-stamps).
(define get-kids-range
  (lambda (hostid area from to)
    (let ((k (find '(kid) '(id)
		   `((= hostid ,hostid) (= kid ,area)
		     (>= ts ,from) (<= ts ,to))
		   '(order-by ((ts ASC))))))
      (if (cursor-null? k)
	  ()
	  (finite-cursor->list (cursor-map cadar k))))))

(define-struct graph (data))

;; Esta funcin regresa (para todo tipo de areas de datos siendo
;; monitoreados) una lista de parejas.  Cada pareja tiene la forma:
;; (etiqueta valor), donde la etiqueta es una cadena que ser impresa
;; en la pgina y valor ser graficado en la grfica.
(define get-data-from
  (lambda (area kid)
    (cond ((string=? area "fs")
	   ;; mountat (punto de montaje) es la etiqueta
	   ;; percent es el valor que estamos graficando
	   (let ((c (find '(fs) '(mountat percent) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (finite-cursor->list (cursor-map (lambda (r)
						    (map cadr r))
						  c)))))
	  ((string=? area "processor")
	   (let ((c (find '(processor) '(load_avg) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (finite-cursor->list (cursor-map (lambda (r)
						    (list "carga" (cadar r)))
						  c)))))

	  ((string=? area "process")
	   (let ((c (find '(process) '(demons system user) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (flatten
		  (finite-cursor->list (cursor-map (lambda (r)
						     (let ((vals (map cadr r)))
						       `(("demonios" ,(list-ref vals 0))
							 ("sistema" ,(list-ref vals 1))
							 ("usuario" ,(list-ref vals 2)))))
						   c))))))
	  
	  ((string=? area "memory")
	   ;; RAM | Swap (las nicas dos etiquetas)
	   ;; Porcentaje usado
	   (let ((c (find '(memory) '(used total s_used swap) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (let* ((row (get-row c))
			(mem-u (cadr (list-ref row 0)))
			(mem-t (cadr (list-ref row 1)))
			(swap-u (cadr (list-ref row 2)))
			(swap-t (cadr (list-ref row 3))))
		   (list
		    (list "RAM" (if (zero? mem-t) 0 (* (/ mem-u mem-t) 100.0)))
		    (list "swap" (if (zero? swap-t) 0 (* (/ swap-u swap-t) 100.0))))))))

	  ((string=? area "users")
	   (let* ((c (find '(users) '("count(*)") `((= id ,kid))))
		  (num (if (cursor-null? c) 0 (cadar (get-row c)))))
	     `(("usuarios" ,num))))

	  ((string=? area "nfs")
	   (let* ((c (find '(nfs) '(imported exported mounted) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (let ((row (get-row c)))
		   `(("importados" ,(cadr (list-ref row 0)))
		     ("exportados" ,(cadr (list-ref row 1)))
		     ("montados" ,(cadr (list-ref row 2))))))))

	  ((string=? area "printer")
	   (let* ((c (find '(printer) '(printer entries) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (finite-cursor->list (cursor-map (lambda (r)
						    (map cadr r))
						  c)))))

	  ((string=? area "network")
	   (let ((c (find '(network) '(interface packets_r packets_t) `((= id ,kid)))))
	     (if (cursor-null? c)
		 '()
		 (flatten
		  (finite-cursor->list (cursor-map (lambda (r)
						     (let ((iface (cadr (list-ref r 0)))
							   (rec (cadr (list-ref r 1)))
							   (trans (cadr (list-ref r 2))))
						       `((,(format "~a-recibidos" iface) rec)
							 (,(format "~a-enviados" iface) trans))))
						   c))))))

	  )))

(define get-raw-data
  (lambda (area label kid)
    (let ((c (void)))
      (cond ((string=? area "fs")
	     ;; mountat (punto de montaje) es la etiqueta
	     ;; percent es el valor que estamos graficando
	     (set! c (find '(fs kid)
			   '((kid ts) (fs used)
			     (fs available) (fs percent))
			   `((= (fs mountat) ,label) (= (fs id) ,kid) (= (fs id) (kid id))))))
	    ((string=? area "processor")
	     (set! c (find '(processor kid)
			   '((kid ts) (processor load_avg) (processor user)
			     (processor system) (processor idle))
			   `((= (processor id) ,kid) (= (processor id) (kid id))))))
	    ((string=? area "process")
	     (set! c (find '(process kid)
			   '((kid ts) (process demons) (process system)
			     (process user) (process run) (process sleep)
			     (process zombie))
			   `((= (process id) ,kid) (= (process id) (kid id))))))
	    ((string=? area "memory")
	     (set! c (find '(memory kid)
			   '((kid ts) (memory used) (memory free) (memory shared)
			     (memory cached) (memory s_used) (memory s_free))
			   `((= (memory id) ,kid) (= (memory id) (kid id))))))
	    ((string=? area "users")
	     (set! c (find '(users kid)
			   '((kid ts) (users user) (users tty) (users remote)
			     (users process) (users login_at))
			   `((= (users id) ,kid) (= (users id) (kid id))))))
	    ((string=? area "nfs")
	     (set! c (find '(nfs kid)
			   '((kid ts) (nfs imported) (nfs exported) (nfs mounted))
			   `((= (nfs id) ,kid) (= (nfs id) (kid id))))))
	    ((string=? area "printer")
	     (set! c (find '(printer kid)
			   '((kid ts) (printer printer) (printer queueing)
			     (printer printing) (printer entries))
			   `((= (printer id) ,kid) (= (printer id) (kid id))))))
	    ((string=? area "network")
	     (set! c (find '(network kid)
			   '((kid ts) (network interface)
			     (network packets_r) (network errors_r) (network drops_r) (network overs_r)
			     (network packets_t) (network errors_t) (network drops_t) (network overs_t))
			   `((= (network id) ,kid) (= (network id) (kid id))))))
	    )
      (if (cursor-null? c)
	  '()
	  (finite-cursor->list (cursor-map (lambda (r)
					     (map (lambda (item)
						    (->string (cadr item)))
						  r))
					   c))))))

(define generate-x-labels
  (lambda (kids)
    (let loop ((kids kids))
      (if (null? kids)
	  ()
	  (let ((c (find '(kid) '(ts) `((= id ,(car kids))))))
	    (cons (timestamp-time (cadar (get-row c))) (loop (cdr kids))))))))

;;; daemon.ss ends here